﻿using AirBomber;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    /// <summary>
    /// Класс для хранения коллекции карт
    /// </summary>
    internal class MapsCollection
    {
        /// <summary>
        /// Словарь (хранилище) с картами
        /// </summary>
        readonly Dictionary<string, MapWithSetAirplanesGeneric<IDrawningObject,
        AbstractMap>> _mapStorages;
        /// <summary>
        /// Возвращение списка названий карт
        /// </summary>
        public List<string> Keys => _mapStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи информации по элементу словаря в файл
        /// </summary>
        private readonly char separatorDict = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char separatorData = ';';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string,
            MapWithSetAirplanesGeneric<IDrawningObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        /// <param name="map">Карта</param>
        public void AddMap(string name, AbstractMap map)
        {
            _mapStorages.Add(name, new(_pictureWidth, _pictureHeight, map));
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        public void DelMap(string name)
        {
            _mapStorages.Remove(name);
        }
        /// <summary>
        /// Доступ к аэродрому
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public MapWithSetAirplanesGeneric<IDrawningObject, AbstractMap> this[string ind]
        {
            get
            {
                _mapStorages.TryGetValue(ind, out var mapWithSetAirplanesGeneric);
                return mapWithSetAirplanesGeneric;
            }
        }

        public IDrawningObject this[string ind, int indDrawningObject]
        {
            get
            {
                _mapStorages.TryGetValue(ind, out var mapWithSetAirplanesGeneric);
                return mapWithSetAirplanesGeneric?.SetAirplanes[indDrawningObject];
            }
        }

        /// <summary>
        /// Сохранение информации по самолетам в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter fs = new(filename))
            {
                fs.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {
                    fs.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }

        public bool SaveMap(string filename, string nameMap)
        {
            if (!_mapStorages.ContainsKey(nameMap))
            {
                return false;
            }
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter fs = new(filename))
            {
                fs.Write($"Map{Environment.NewLine}");
                fs.Write($"Name: {nameMap}{Environment.NewLine}");
                fs.Write(_mapStorages[nameMap].GetData(separatorDict, separatorData));
            }
            return true;
        }

        private void LoadMap(string nameMapStorage, string nameMap, string lineData)
        {
            AbstractMap map = null;
            switch (nameMap)
            {
                case "SimpleMap":
                    map = new SimpleMap();
                    break;
                case "WallMap":
                    map = new WallMap();
                    break;
            }
            if (_mapStorages.ContainsKey(nameMapStorage))
            {
                _mapStorages[nameMapStorage].Clear();
            }
            else
            {
                _mapStorages[nameMapStorage] = new(_pictureWidth, _pictureHeight, map);
            }
            _mapStorages[nameMapStorage].LoadData(lineData.Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
        }

        /// <summary>
        /// Загрузка информации по самолетам в ангарах из файла
        /// </summary>
        /// <param name="filename"></param>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader fs = new(filename))
            {
                var formatData = fs.ReadLine() ?? string.Empty;
                bool isNotMapsCollection = !formatData.Contains("MapsCollection");
                if (formatData.Contains("Map") && isNotMapsCollection)
                {
                    var nameMap = fs.ReadLine().Replace("Name: ", "");
                    var elem = fs.ReadLine().Split(separatorDict);
                    LoadMap(nameMap, elem[0], elem[1]);
                }
                if (isNotMapsCollection)
                {
                    //если нет такой записи, то это не те данные
                    throw new FileFormatException("Формат данных в файле не правильный");
                }
                //очищаем записи
                _mapStorages.Clear();
                while (!fs.EndOfStream)
                {
                    var elem = fs.ReadLine().Split(separatorDict);
                    LoadMap(elem[0], elem[1], elem[2]);
                }
            }
        }
    }
}
